from os import getgid, getuid
from pathlib import Path
from pwd import getpwuid
from secrets import token_urlsafe
from typing import Optional

from typer import Option

from kolombo.console import debug, enable_debug, finished, info, started, step
from kolombo.util import run

_virtual_configs = {
    "addresses": "bob@example.com bob@example.com",
    "domains": "example.com",
    "mailbox": "bob@example.com bob@example.com/",
    "ssl_map": (
        "example.com /etc/letsencrypt/live/example.com/privkey.pem "
        "/etc/letsencrypt/live/example.com/fullchain.pem"
    ),
}


def init(
    debug_mode: bool = Option(  # noqa: B008
        False,
        "--debug",
        "-D",
        help="Enable debug mode?",
        is_flag=True,
    ),
    nginx_secret_key: Optional[str] = Option(  # noqa: B008
        None,
        "--nginx-secret",
        help="Secret key for communication between NginX and auth API",
    ),
    max_auth_attempts: int = Option(  # noqa: B008
        3,
        "--max-auth",
        help="Maximum auth attempts per one session",
        min=1,
    ),
    passwords_salt: Optional[str] = Option(  # noqa: B008
        None,
        "--salt",
        help="Secret key to be used as salt for passwords hashing",
    ),
) -> None:
    from kolombo import __version__ as kolombo_version

    username = getpwuid(getuid()).pw_name
    started(f"Setting up Kolombo for current user [b]{username}[/]")

    step("Creating /etc/kolombo folder ([u]need root privileges[/])")

    info("Creating /etc/kolombo folder (as root)")
    run(["mkdir", "-p", "-m", "750", "/etc/kolombo"], as_root=True)
    info(f"Changing /etc/kolombo owner to {username} (as root)")
    run(["chown", f"{getuid()}:{getgid()}", "/etc/kolombo"], as_root=True)

    step("Writing configuration to /etc/kolombo/kolombo.conf")
    if debug_mode:
        enable_debug()

    if nginx_secret_key is None:
        nginx_secret_key = token_urlsafe(20)

    if passwords_salt is None:
        passwords_salt = token_urlsafe(20)

    configuration = (
        f"### Generated by kolombo v{kolombo_version}\n\n"
        "# Whether debug mode is enabled (0 - disabled, 1 - enabled)\n"
        f"DEBUG={int(debug_mode)}\n"
        "# Secret key that is used to determine that nginx is using API\n"
        f"NGINX_SECRET_KEY={nginx_secret_key}\n"
        "# Maximum auth attempts per one session\n"
        f"MAX_ATTEMPTS={max_auth_attempts}\n"
        "# Salt used for passwords hashing\n"
        f"SALT={passwords_salt}\n"
    )
    with open("/etc/kolombo/kolombo.conf", "w") as config_file:
        config_file.write(configuration)

    step("Populating /etc/kolombo with default folders and files")
    debug("Creating /etc/kolombo folders for volumes")
    folders = ("maildirs", "mail-enabled", "virtual", "dkim_keys")
    for folder in folders:
        Path(f"/etc/kolombo/{folder}").mkdir(mode=0o770, exist_ok=True)

    for file in ("addresses", "domains", "mailbox", "ssl_map"):
        debug(f"Writing default file to /etc/kolombo/virtual/{file}")
        with open(f"/etc/kolombo/virtual/{file}", "w") as virtual_file:
            virtual_file.write(f"# {_virtual_configs[file]}\n")

    finished("Kolombo is set up!")
